=== modified file 'NEWS'
--- NEWS	2009-08-05 17:04:44 +0000
+++ NEWS	2009-08-06 02:23:37 +0000
@@ -146,7 +146,6 @@
   the same way that sending Ctrl-\\ does on other platforms.
   (John Arbash Meinel)
 
-
 Documentation
 *************
 
@@ -172,6 +171,9 @@
   UIFactory. 
   (Martin Pool)
 
+* ``WorkingTree._check`` now requires a references dict with keys matching
+  those returned by ``WorkingTree._get_check_refs``. (Robert Collins)
+
 Internals
 *********
 
@@ -902,6 +904,9 @@
   cause mismatched physical locks to cause test errors rather than just
   reporting to the screen. (Robert Collins)
 
+* -Dprogress will cause pdb to start up if a progress view jumps
+  backwards. (Robert Collins)
+
 * Fallback ``CredentialStore`` instances registered with ``fallback=True``
   are now be able to provide credentials if obtaining credentials 
   via ~/.bazaar/authentication.conf fails. (Jelmer Vernooij, 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2009-07-10 08:42:35 +0000
+++ bzrlib/branch.py	2009-08-04 04:36:34 +0000
@@ -149,6 +149,14 @@
         if self._partial_revision_history_cache[-1] == _mod_revision.NULL_REVISION:
             self._partial_revision_history_cache.pop()
 
+    def _get_check_refs(self):
+        """Get the references needed for check().
+
+        See bzrlib.check.
+        """
+        revid = self.last_revision()
+        return [('revision-existence', revid), ('lefthand-distance', revid)]
+
     @staticmethod
     def open(base, _unsupported=False, possible_transports=None):
         """Open the branch rooted at base.
@@ -1212,7 +1220,7 @@
         target._set_all_reference_info(target_reference_dict)
 
     @needs_read_lock
-    def check(self):
+    def check(self, refs):
         """Check consistency of the branch.
 
         In particular this checks that revisions given in the revision-history
@@ -1221,42 +1229,23 @@
 
         Callers will typically also want to check the repository.
 
+        :param refs: Calculated refs for this branch as specified by
+            branch._get_check_refs()
         :return: A BranchCheckResult.
         """
-        ret = BranchCheckResult(self)
-        mainline_parent_id = None
+        result = BranchCheckResult(self)
         last_revno, last_revision_id = self.last_revision_info()
-        real_rev_history = []
-        try:
-            for revid in self.repository.iter_reverse_revision_history(
-                last_revision_id):
-                real_rev_history.append(revid)
-        except errors.RevisionNotPresent:
-            ret.ghosts_in_mainline = True
-        else:
-            ret.ghosts_in_mainline = False
-        real_rev_history.reverse()
-        if len(real_rev_history) != last_revno:
-            raise errors.BzrCheckError('revno does not match len(mainline)'
-                ' %s != %s' % (last_revno, len(real_rev_history)))
-        # TODO: We should probably also check that real_rev_history actually
-        #       matches self.revision_history()
-        for revision_id in real_rev_history:
-            try:
-                revision = self.repository.get_revision(revision_id)
-            except errors.NoSuchRevision, e:
-                raise errors.BzrCheckError("mainline revision {%s} not in repository"
-                            % revision_id)
-            # In general the first entry on the revision history has no parents.
-            # But it's not illegal for it to have parents listed; this can happen
-            # in imports from Arch when the parents weren't reachable.
-            if mainline_parent_id is not None:
-                if mainline_parent_id not in revision.parent_ids:
-                    raise errors.BzrCheckError("previous revision {%s} not listed among "
-                                        "parents of {%s}"
-                                        % (mainline_parent_id, revision_id))
-            mainline_parent_id = revision_id
-        return ret
+        actual_revno = refs[('lefthand-distance', last_revision_id)]
+        if actual_revno != last_revno:
+            result.errors.append(errors.BzrCheckError(
+                'revno does not match len(mainline) %s != %s' % (
+                last_revno, actual_revno)))
+        # TODO: We should probably also check that self.revision_history
+        # matches the repository for older branch formats.
+        # If looking for the code that cross-checks repository parents against
+        # the iter_reverse_revision_history output, that is now a repository
+        # specific check.
+        return result
 
     def _get_checkout_format(self):
         """Return the most suitable metadir for a checkout of this branch.
@@ -2839,7 +2828,7 @@
 
     def __init__(self, branch):
         self.branch = branch
-        self.ghosts_in_mainline = False
+        self.errors = []
 
     def report_results(self, verbose):
         """Report the check results via trace.note.
@@ -2847,11 +2836,10 @@
         :param verbose: Requests more detailed display of what was checked,
             if any.
         """
-        note('checked branch %s format %s',
-             self.branch.base,
-             self.branch._format)
-        if self.ghosts_in_mainline:
-            note('branch contains ghosts in mainline')
+        note('checked branch %s format %s', self.branch.base,
+            self.branch._format)
+        for error in self.errors:
+            note('found error:%s', error)
 
 
 class Converter5to6(object):

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2009-03-23 14:59:43 +0000
+++ bzrlib/check.py	2009-08-05 02:30:59 +0000
@@ -32,6 +32,20 @@
 # raising them.  If there's more than one exception it'd be good to see them
 # all.
 
+"""Checking of bzr objects.
+
+check_refs is a concept used for optimising check. Objects that depend on other
+objects (e.g. tree on repository) can list the objects they would be requesting
+so that when the dependent object is checked, matches can be pulled out and
+evaluated in-line rather than re-reading the same data many times.
+check_refs are tuples (kind, value). Currently defined kinds are:
+* 'trees', where value is a revid and the looked up objects are revision trees.
+* 'lefthand-distance', where value is a revid and the looked up objects are the
+  distance along the lefthand path to NULL for that revid.
+* 'revision-existence', where value is a revid, and the result is True or False
+  indicating that the revision was found/not found.
+"""
+
 from bzrlib import errors, osutils
 from bzrlib import repository as _mod_repository
 from bzrlib import revision
@@ -39,6 +53,7 @@
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import BzrCheckError
 from bzrlib.repository import Repository
+from bzrlib.revision import NULL_REVISION
 from bzrlib.symbol_versioning import deprecated_function, deprecated_in
 from bzrlib.trace import log_error, note
 import bzrlib.ui
@@ -49,76 +64,145 @@
 
     # The Check object interacts with InventoryEntry.check, etc.
 
-    def __init__(self, repository):
+    def __init__(self, repository, check_repo=True):
         self.repository = repository
-        self.checked_text_cnt = 0
         self.checked_rev_cnt = 0
-        self.ghosts = []
-        self.repeated_text_cnt = 0
+        self.ghosts = set()
         self.missing_parent_links = {}
         self.missing_inventory_sha_cnt = 0
         self.missing_revision_cnt = 0
-        # maps (file-id, version) -> sha1; used by InventoryFile._check
-        self.checked_texts = {}
         self.checked_weaves = set()
         self.unreferenced_versions = set()
         self.inconsistent_parents = []
         self.rich_roots = repository.supports_rich_root()
         self.text_key_references = {}
+        self.check_repo = check_repo
+        self.other_results = []
+        # Plain text lines to include in the report
+        self._report_items = []
+        # Keys we are looking for; may be large and need spilling to disk.
+        # key->(type(revision/inventory/text/signature/map), sha1, first-referer)
+        self.pending_keys = {}
+        # Ancestors map for all of revisions being checked; while large helper
+        # functions we call would create it anyway, so better to have once and
+        # keep.
+        self.ancestors = {}
 
-    def check(self):
+    def check(self, callback_refs=None, check_repo=True):
+        if callback_refs is None:
+            callback_refs = {}
         self.repository.lock_read()
         self.progress = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
-            self.progress.update('retrieving inventory', 0, 2)
-            # do not put in init, as it should be done with progess,
-            # and inside the lock.
-            self.inventory_weave = self.repository.inventories
-            self.progress.update('checking revision graph', 1)
-            self.check_revision_graph()
-            self.plan_revisions()
-            revno = 0
-            while revno < len(self.planned_revisions):
-                rev_id = self.planned_revisions[revno]
-                self.progress.update('checking revision', revno,
-                                     len(self.planned_revisions))
-                revno += 1
-                self.check_one_rev(rev_id)
-            # check_weaves is done after the revision scan so that
-            # revision index is known to be valid.
-            self.check_weaves()
+            self.progress.update('check', 0, 4)
+            if self.check_repo:
+                self.progress.update('checking revisions', 0)
+                self.check_revisions()
+                self.progress.update('checking commit contents', 1)
+                self.repository._check_inventories(self)
+                self.progress.update('checking file graphs', 2)
+                # check_weaves is done after the revision scan so that
+                # revision index is known to be valid.
+                self.check_weaves()
+            self.progress.update('checking branches and trees', 3)
+            if callback_refs:
+                repo = self.repository
+                # calculate all refs, and callback the objects requesting them.
+                refs = {}
+                wanting_items = set()
+                # Current crude version calculates everything and calls
+                # everything at once. Doing a queue and popping as things are
+                # satisfied would be cheaper on memory [but few people have
+                # huge numbers of working trees today. TODO: fix before
+                # landing].
+                distances = set()
+                existences = set()
+                for ref, wantlist in callback_refs.iteritems():
+                    wanting_items.update(wantlist)
+                    kind, value = ref
+                    if kind == 'trees':
+                        refs[ref] = repo.revision_tree(value)
+                    elif kind == 'lefthand-distance':
+                        distances.add(value)
+                    elif kind == 'revision-existence':
+                        existences.add(value)
+                    else:
+                        raise AssertionError(
+                            'unknown ref kind for ref %s' % ref)
+                node_distances = repo.get_graph().find_lefthand_distances(distances)
+                for key, distance in node_distances.iteritems():
+                    refs[('lefthand-distance', key)] = distance
+                    if key in existences and distance > 0:
+                        refs[('revision-existence', key)] = True
+                        existences.remove(key)
+                parent_map = repo.get_graph().get_parent_map(existences)
+                for key in parent_map:
+                    refs[('revision-existence', key)] = True
+                    existences.remove(key)
+                for key in existences:
+                    refs[('revision-existence', key)] = False
+                for item in wanting_items:
+                    if isinstance(item, WorkingTree):
+                        item._check(refs)
+                    if isinstance(item, Branch):
+                        self.other_results.append(item.check(refs))
         finally:
             self.progress.finished()
             self.repository.unlock()
 
-    def check_revision_graph(self):
+    def _check_revisions(self, revisions_iterator):
+        """Check revision objects by decorating a generator.
+
+        :param revisions_iterator: An iterator of(revid, Revision-or-None).
+        :return: A generator of the contents of revisions_iterator.
+        """
+        self.planned_revisions = set()
+        for revid, revision in revisions_iterator:
+            yield revid, revision
+            self._check_one_rev(revid, revision)
+        # Flatten the revisions we found to guarantee consistent later
+        # iteration.
+        self.planned_revisions = list(self.planned_revisions)
+        # TODO: extract digital signatures as items to callback on too.
+
+    def check_revisions(self):
+        """Scan revisions, checking data directly available as we go."""
+        revision_iterator = self.repository._iter_revisions(None)
+        revision_iterator = self._check_revisions(revision_iterator)
+        # We read the all revisions here:
+        # - doing this allows later code to depend on the revision index.
+        # - we can fill out existence flags at this point
+        # - we can read the revision inventory sha at this point
+        # - we can check properties and serialisers etc.
         if not self.repository.revision_graph_can_have_wrong_parents():
-            # This check is not necessary.
+            # The check against the index isn't needed.
             self.revs_with_bad_parents_in_index = None
-            return
-        bad_revisions = self.repository._find_inconsistent_revision_parents()
-        self.revs_with_bad_parents_in_index = list(bad_revisions)
-
-    def plan_revisions(self):
-        repository = self.repository
-        self.planned_revisions = repository.all_revision_ids()
-        self.progress.clear()
-        inventoried = set(key[-1] for key in self.inventory_weave.keys())
-        awol = set(self.planned_revisions) - inventoried
-        if len(awol) > 0:
-            raise BzrCheckError('Stored revisions missing from inventory'
-                '{%s}' % ','.join([f for f in awol]))
+            for thing in revision_iterator:
+                pass
+        else:
+            bad_revisions = self.repository._find_inconsistent_revision_parents(
+                revision_iterator)
+            self.revs_with_bad_parents_in_index = list(bad_revisions)
 
     def report_results(self, verbose):
+        if self.check_repo:
+            self._report_repo_results(verbose)
+        for result in self.other_results:
+            result.report_results(verbose)
+
+    def _report_repo_results(self, verbose):
         note('checked repository %s format %s',
              self.repository.bzrdir.root_transport,
              self.repository._format)
         note('%6d revisions', self.checked_rev_cnt)
         note('%6d file-ids', len(self.checked_weaves))
-        note('%6d unique file texts', self.checked_text_cnt)
-        note('%6d repeated file texts', self.repeated_text_cnt)
-        note('%6d unreferenced text versions',
-             len(self.unreferenced_versions))
+        if verbose:
+            note('%6d unreferenced text versions',
+                len(self.unreferenced_versions))
+        if verbose and len(self.unreferenced_versions):
+                for file_id, revision_id in self.unreferenced_versions:
+                    log_error('unreferenced version: {%s} in %s', revision_id,
+                        file_id)
         if self.missing_inventory_sha_cnt:
             note('%6d revisions are missing inventory_sha1',
                  self.missing_inventory_sha_cnt)
@@ -138,10 +222,6 @@
                     note('      %s should be in the ancestry for:', link)
                     for linker in linkers:
                         note('       * %s', linker)
-            if verbose:
-                for file_id, revision_id in self.unreferenced_versions:
-                    log_error('unreferenced version: {%s} in %s', revision_id,
-                        file_id)
         if len(self.inconsistent_parents):
             note('%6d inconsistent parents', len(self.inconsistent_parents))
             if verbose:
@@ -161,60 +241,75 @@
                         '       %s has wrong parents in index: '
                         '%r should be %r',
                         revision_id, index_parents, actual_parents)
-
-    def check_one_rev(self, rev_id):
-        """Check one revision.
-
-        rev_id - the one to check
+        for item in self._report_items:
+            note(item)
+
+    def _check_one_rev(self, rev_id, rev):
+        """Cross-check one revision.
+
+        :param rev_id: A revision id to check.
+        :param rev: A revision or None to indicate a missing revision.
         """
-        rev = self.repository.get_revision(rev_id)
-
         if rev.revision_id != rev_id:
-            raise BzrCheckError('wrong internal revision id in revision {%s}'
-                                % rev_id)
-
+            self._report_items.append(
+                'Mismatched internal revid {%s} and index revid {%s}' % (
+                rev.revision_id, rev_id))
+            rev_id = rev.revision_id
+        # Check this revision tree etc, and count as seen when we encounter a
+        # reference to it.
+        self.planned_revisions.add(rev_id)
+        # It is not a ghost
+        self.ghosts.discard(rev_id)
+        # Count all parents as ghosts if we haven't seen them yet.
         for parent in rev.parent_ids:
             if not parent in self.planned_revisions:
-                # rev has a parent we didn't know about.
-                missing_links = self.missing_parent_links.get(parent, [])
-                missing_links.append(rev_id)
-                self.missing_parent_links[parent] = missing_links
-                # list based so somewhat slow,
-                # TODO have a planned_revisions list and set.
-                if self.repository.has_revision(parent):
-                    missing_ancestry = self.repository.get_ancestry(parent)
-                    for missing in missing_ancestry:
-                        if (missing is not None
-                            and missing not in self.planned_revisions):
-                            self.planned_revisions.append(missing)
-                else:
-                    self.ghosts.append(rev_id)
-
-        if rev.inventory_sha1:
-            # Loopback - this is currently circular logic as the
-            # knit get_inventory_sha1 call returns rev.inventory_sha1.
-            # Repository.py's get_inventory_sha1 should instead return
-            # inventories.get_record_stream([(revid,)]).next().sha1 or
-            # similar.
-            inv_sha1 = self.repository.get_inventory_sha1(rev_id)
-            if inv_sha1 != rev.inventory_sha1:
-                raise BzrCheckError('Inventory sha1 hash doesn\'t match'
-                    ' value in revision {%s}' % rev_id)
-        self._check_revision_tree(rev_id)
+                self.ghosts.add(parent)
+        
+        self.ancestors[rev_id] = tuple(rev.parent_ids) or (NULL_REVISION,)
+        self.add_pending_item(rev_id, ('inventories', rev_id), 'inventory',
+            rev.inventory_sha1)
         self.checked_rev_cnt += 1
 
+    def add_pending_item(self, referer, key, kind, sha1):
+        """Add a reference to a sha1 to be cross checked against a key.
+
+        :param referer: The referer that expects key to have sha1.
+        :param key: A storage key e.g. ('texts', 'foo@bar-20040504-1234')
+        :param kind: revision/inventory/text/map/signature
+        :param sha1: A hex sha1 or None if no sha1 is known.
+        """
+        existing = self.pending_keys.get(key)
+        if existing:
+            if sha1 != existing[1]:
+                self._report_items.append('Multiple expected sha1s for %s. {%s}'
+                    ' expects {%s}, {%s} expects {%s}', (
+                    key, referer, sha1, existing[1], existing[0]))
+        else:
+            self.pending_keys[key] = (kind, sha1, referer)
+
     def check_weaves(self):
         """Check all the weaves we can get our hands on.
         """
         weave_ids = []
-        self.progress.update('checking inventory', 0, 2)
-        self.inventory_weave.check(progress_bar=self.progress)
-        self.progress.update('checking text storage', 1, 2)
-        self.repository.texts.check(progress_bar=self.progress)
-        weave_checker = self.repository._get_versioned_file_checker(
-            text_key_references=self.text_key_references)
+        storebar = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            self._check_weaves(storebar)
+        finally:
+            storebar.finished()
+
+    def _check_weaves(self, storebar):
+        storebar.update('text-index', 0, 2)
+        if self.repository._format.fast_deltas:
+            # We haven't considered every fileid instance so far.
+            weave_checker = self.repository._get_versioned_file_checker(
+                ancestors=self.ancestors)
+        else:
+            weave_checker = self.repository._get_versioned_file_checker(
+                text_key_references=self.text_key_references,
+                ancestors=self.ancestors)
+        storebar.update('file-graph', 1)
         result = weave_checker.check_file_version_parents(
-            self.repository.texts, progress_bar=self.progress)
+            self.repository.texts)
         self.checked_weaves = weave_checker.file_ids
         bad_parents, unused_versions = result
         bad_parents = bad_parents.items()
@@ -228,28 +323,8 @@
                 (revision_id, weave_id, weave_parents, correct_parents))
         self.unreferenced_versions.update(unused_versions)
 
-    def _check_revision_tree(self, rev_id):
-        tree = self.repository.revision_tree(rev_id)
-        inv = tree.inventory
-        seen_ids = set()
-        seen_names = set()
-        for path, ie in inv.iter_entries():
-            self._add_entry_to_text_key_references(inv, ie)
-            file_id = ie.file_id
-            if file_id in seen_ids:
-                raise BzrCheckError('duplicated file_id {%s} '
-                                    'in inventory for revision {%s}'
-                                    % (file_id, rev_id))
-            seen_ids.add(file_id)
-            ie.check(self, rev_id, inv, tree)
-            if path in seen_names:
-                raise BzrCheckError('duplicated path %s '
-                                    'in inventory for revision {%s}'
-                                    % (path, rev_id))
-            seen_names.add(path)
-
     def _add_entry_to_text_key_references(self, inv, entry):
-        if not self.rich_roots and entry == inv.root:
+        if not self.rich_roots and entry.name == '':
             return
         key = (entry.file_id, entry.revision)
         self.text_key_references.setdefault(key, False)
@@ -263,13 +338,14 @@
 
     Results are reported through logging.
 
-    Deprecated in 1.6.  Please use check_branch instead.
+    Deprecated in 1.6.  Please use check_dwim instead.
 
     :raise BzrCheckError: if there's a consistency error.
     """
     check_branch(branch, verbose)
 
 
+@deprecated_function(deprecated_in((1,16,0)))
 def check_branch(branch, verbose):
     """Run consistency checks on a branch.
 
@@ -279,56 +355,113 @@
     """
     branch.lock_read()
     try:
-        branch_result = branch.check()
+        needed_refs = {}
+        for ref in branch._get_check_refs():
+            needed_refs.setdefault(ref, []).append(branch)
+        result = branch.repository.check([branch.last_revision()], needed_refs)
+        branch_result = result.other_results[0]
     finally:
         branch.unlock()
     branch_result.report_results(verbose)
 
 
+def scan_branch(branch, needed_refs, to_unlock):
+    """Scan a branch for refs.
+
+    :param branch:  The branch to schedule for checking.
+    :param needed_refs: Refs we are accumulating.
+    :param to_unlock: The unlock list accumulating.
+    """
+    note("Checking branch at '%s'." % (branch.base,))
+    branch.lock_read()
+    to_unlock.append(branch)
+    branch_refs = branch._get_check_refs()
+    for ref in branch_refs:
+        reflist = needed_refs.setdefault(ref, [])
+        reflist.append(branch)
+
+
+def scan_tree(base_tree, tree, needed_refs, to_unlock):
+    """Scan a tree for refs.
+
+    :param base_tree: The original tree check opened, used to detect duplicate
+        tree checks.
+    :param tree:  The tree to schedule for checking.
+    :param needed_refs: Refs we are accumulating.
+    :param to_unlock: The unlock list accumulating.
+    """
+    if base_tree is not None and tree.basedir == base_tree.basedir:
+        return
+    note("Checking working tree at '%s'." % (tree.basedir,))
+    tree.lock_read()
+    to_unlock.append(tree)
+    tree_refs = tree._get_check_refs()
+    for ref in tree_refs:
+        reflist = needed_refs.setdefault(ref, [])
+        reflist.append(tree)
+
+
 def check_dwim(path, verbose, do_branch=False, do_repo=False, do_tree=False):
+    """Check multiple objects.
+
+    If errors occur they are accumulated and reported as far as possible, and
+    an exception raised at the end of the process.
+    """
     try:
-        tree, branch, repo, relpath = \
+        base_tree, branch, repo, relpath = \
                         BzrDir.open_containing_tree_branch_or_repository(path)
     except errors.NotBranchError:
-        tree = branch = repo = None
-
-    if do_tree:
-        if tree is not None:
-            note("Checking working tree at '%s'."
-                 % (tree.bzrdir.root_transport.base,))
-            tree._check()
-        else:
-            log_error("No working tree found at specified location.")
-
-    if branch is not None:
-        # We have a branch
-        if repo is None:
-            # The branch is in a shared repository
-            repo = branch.repository
-        branches = [branch]
-    elif repo is not None:
-        branches = repo.find_branches(using=True)
-
-    if repo is not None:
-        repo.lock_read()
-        try:
-            if do_repo:
-                note("Checking repository at '%s'."
-                     % (repo.bzrdir.root_transport.base,))
-                result = repo.check()
+        base_tree = branch = repo = None
+
+    to_unlock = []
+    needed_refs= {}
+    try:
+        if base_tree is not None:
+            # If the tree is a lightweight checkout we won't see it in
+            # repo.find_branches - add now.
+            if do_tree:
+                scan_tree(None, base_tree, needed_refs, to_unlock)
+            branch = base_tree.branch
+        if branch is not None:
+            # We have a branch
+            if repo is None:
+                # The branch is in a shared repository
+                repo = branch.repository
+        if repo is not None:
+            repo.lock_read()
+            to_unlock.append(repo)
+            branches = repo.find_branches(using=True)
+            saw_tree = False
+            if do_branch or do_tree:
+                for branch in branches:
+                    if do_tree:
+                        try:
+                            tree = branch.bzrdir.open_workingtree()
+                            saw_tree = True
+                        except (errors.NotLocalUrl, errors.NoWorkingTree):
+                            pass
+                        else:
+                            scan_tree(base_tree, tree, needed_refs, to_unlock)
+                    if do_branch:
+                        scan_branch(branch, needed_refs, to_unlock)
+            if do_branch and not branches:
+                log_error("No branch found at specified location.")
+            if do_tree and base_tree is None and not saw_tree:
+                log_error("No working tree found at specified location.")
+            if do_repo or do_branch or do_tree:
+                if do_repo:
+                    note("Checking repository at '%s'."
+                         % (repo.bzrdir.root_transport.base,))
+                result = repo.check(None, callback_refs=needed_refs,
+                    check_repo=do_repo)
                 result.report_results(verbose)
+        else:
+            if do_tree:
+                log_error("No working tree found at specified location.")
             if do_branch:
-                if branches == []:
-                    log_error("No branch found at specified location.")
-                else:
-                    for branch in branches:
-                        note("Checking branch at '%s'."
-                             % (branch.bzrdir.root_transport.base,))
-                        check_branch(branch, verbose)
-        finally:
-            repo.unlock()
-    else:
-        if do_branch:
-            log_error("No branch found at specified location.")
-        if do_repo:
-            log_error("No repository found at specified location.")
+                log_error("No branch found at specified location.")
+            if do_repo:
+                log_error("No repository found at specified location.")
+    finally:
+        for thing in to_unlock:
+            thing.unlock()

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2009-06-12 18:05:15 +0000
+++ bzrlib/graph.py	2009-08-04 04:36:34 +0000
@@ -311,6 +311,27 @@
         # get there.
         return known_revnos[cur_tip] + num_steps
 
+    def find_lefthand_distances(self, keys):
+        """Find the distance to null for all the keys in keys.
+
+        :param keys: keys to lookup.
+        :return: A dict key->distance for all of keys.
+        """
+        # Optimisable by concurrent searching, but a random spread should get
+        # some sort of hit rate.
+        result = {}
+        known_revnos = []
+        ghosts = []
+        for key in keys:
+            try:
+                known_revnos.append(
+                    (key, self.find_distance_to_null(key, known_revnos)))
+            except errors.GhostRevisionsHaveNoRevno:
+                ghosts.append(key)
+        for key in ghosts:
+            known_revnos.append((key, -1))
+        return dict(known_revnos)
+
     def find_unique_ancestors(self, unique_revision, common_revisions):
         """Find the unique ancestors for a revision versus others.
 

=== modified file 'bzrlib/groupcompress.py'
--- bzrlib/groupcompress.py	2009-07-06 20:21:34 +0000
+++ bzrlib/groupcompress.py	2009-08-04 04:36:34 +0000
@@ -1075,11 +1075,14 @@
     def get_annotator(self):
         return annotate.Annotator(self)
 
-    def check(self, progress_bar=None):
+    def check(self, progress_bar=None, keys=None):
         """See VersionedFiles.check()."""
-        keys = self.keys()
-        for record in self.get_record_stream(keys, 'unordered', True):
-            record.get_bytes_as('fulltext')
+        if keys is None:
+            keys = self.keys()
+            for record in self.get_record_stream(keys, 'unordered', True):
+                record.get_bytes_as('fulltext')
+        else:
+            return self.get_record_stream(keys, 'unordered', True)
 
     def _check_add(self, key, lines, random_id, check_content):
         """check that version_id and lines are safe to add."""

=== modified file 'bzrlib/help_topics/en/debug-flags.txt'
--- bzrlib/help_topics/en/debug-flags.txt	2009-07-24 03:15:56 +0000
+++ bzrlib/help_topics/en/debug-flags.txt	2009-08-04 04:36:34 +0000
@@ -22,6 +22,7 @@
 -Dindex           Trace major index operations.
 -Dknit            Trace knit operations.
 -Dlock            Trace when lockdir locks are taken or released.
+-Dprogress        Trace progress bar operations.
 -Dmerge           Emit information for debugging merges.
 -Dunlock          Some errors during unlock are treated as warnings.
 -Dpack            Emit information about pack operations.

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2009-07-20 05:19:53 +0000
+++ bzrlib/inventory.py	2009-08-05 02:30:59 +0000
@@ -260,7 +260,7 @@
     def versionable_kind(kind):
         return (kind in ('file', 'directory', 'symlink', 'tree-reference'))
 
-    def check(self, checker, rev_id, inv, tree):
+    def check(self, checker, rev_id, inv):
         """Check this inventory entry is intact.
 
         This is a template method, override _check for kind specific
@@ -272,18 +272,18 @@
         :param rev_id: Revision id from which this InventoryEntry was loaded.
              Not necessarily the last-changed revision for this file.
         :param inv: Inventory from which the entry was loaded.
-        :param tree: RevisionTree for this entry.
         """
         if self.parent_id is not None:
             if not inv.has_id(self.parent_id):
                 raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
                         % (self.parent_id, rev_id))
-        self._check(checker, rev_id, tree)
+        checker._add_entry_to_text_key_references(inv, self)
+        self._check(checker, rev_id)
 
-    def _check(self, checker, rev_id, tree):
+    def _check(self, checker, rev_id):
         """Check this inventory entry for kind specific errors."""
-        raise BzrCheckError('unknown entry kind %r in revision {%s}' %
-                            (self.kind, rev_id))
+        checker._report_items.append(
+            'unknown entry kind %r in revision {%s}' % (self.kind, rev_id))
 
     def copy(self):
         """Clone this inventory entry."""
@@ -402,7 +402,7 @@
                  'text_id', 'parent_id', 'children', 'executable',
                  'revision', 'symlink_target', 'reference_revision']
 
-    def _check(self, checker, rev_id, tree):
+    def _check(self, checker, rev_id):
         """See InventoryEntry._check"""
 
     def __init__(self, file_id):
@@ -431,11 +431,16 @@
                  'text_id', 'parent_id', 'children', 'executable',
                  'revision', 'symlink_target', 'reference_revision']
 
-    def _check(self, checker, rev_id, tree):
+    def _check(self, checker, rev_id):
         """See InventoryEntry._check"""
-        if self.text_sha1 is not None or self.text_size is not None or self.text_id is not None:
-            raise BzrCheckError('directory {%s} has text in revision {%s}'
+        if (self.text_sha1 is not None or self.text_size is not None or
+            self.text_id is not None):
+            checker._report_items.append('directory {%s} has text in revision {%s}'
                                 % (self.file_id, rev_id))
+        # Directories are stored as ''.
+        checker.add_pending_item(rev_id,
+            ('texts', self.file_id, self.revision), 'text',
+             'da39a3ee5e6b4b0d3255bfef95601890afd80709')
 
     def copy(self):
         other = InventoryDirectory(self.file_id, self.name, self.parent_id)
@@ -474,27 +479,16 @@
                  'text_id', 'parent_id', 'children', 'executable',
                  'revision', 'symlink_target', 'reference_revision']
 
-    def _check(self, checker, tree_revision_id, tree):
+    def _check(self, checker, tree_revision_id):
         """See InventoryEntry._check"""
-        key = (self.file_id, self.revision)
-        if key in checker.checked_texts:
-            prev_sha = checker.checked_texts[key]
-            if prev_sha != self.text_sha1:
-                raise BzrCheckError(
-                    'mismatched sha1 on {%s} in {%s} (%s != %s) %r' %
-                    (self.file_id, tree_revision_id, prev_sha, self.text_sha1,
-                     t))
-            else:
-                checker.repeated_text_cnt += 1
-                return
-
-        checker.checked_text_cnt += 1
-        # We can't check the length, because Weave doesn't store that
-        # information, and the whole point of looking at the weave's
-        # sha1sum is that we don't have to extract the text.
-        if (self.text_sha1 != tree._repository.texts.get_sha1s([key])[key]):
-            raise BzrCheckError('text {%s} version {%s} wrong sha1' % key)
-        checker.checked_texts[key] = self.text_sha1
+        # TODO: check size too.
+        checker.add_pending_item(tree_revision_id,
+            ('texts', self.file_id, self.revision), 'text',
+             self.text_sha1)
+        if self.text_size is None:
+            checker._report_items.append(
+                'fileid {%s} in {%s} has None for text_size' % (self.file_id,
+                tree_revision_id))
 
     def copy(self):
         other = InventoryFile(self.file_id, self.name, self.parent_id)
@@ -598,14 +592,20 @@
                  'text_id', 'parent_id', 'children', 'executable',
                  'revision', 'symlink_target', 'reference_revision']
 
-    def _check(self, checker, rev_id, tree):
+    def _check(self, checker, tree_revision_id):
         """See InventoryEntry._check"""
         if self.text_sha1 is not None or self.text_size is not None or self.text_id is not None:
-            raise BzrCheckError('symlink {%s} has text in revision {%s}'
-                    % (self.file_id, rev_id))
+            checker._report_items.append(
+               'symlink {%s} has text in revision {%s}'
+                    % (self.file_id, tree_revision_id))
         if self.symlink_target is None:
-            raise BzrCheckError('symlink {%s} has no target in revision {%s}'
-                    % (self.file_id, rev_id))
+            checker._report_items.append(
+                'symlink {%s} has no target in revision {%s}'
+                    % (self.file_id, tree_revision_id))
+        # Symlinks are stored as ''
+        checker.add_pending_item(tree_revision_id,
+            ('texts', self.file_id, self.revision), 'text',
+             'da39a3ee5e6b4b0d3255bfef95601890afd80709')
 
     def copy(self):
         other = InventoryLink(self.file_id, self.name, self.parent_id)

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2009-07-28 08:09:13 +0000
+++ bzrlib/knit.py	2009-08-04 04:36:34 +0000
@@ -1045,8 +1045,15 @@
     def get_annotator(self):
         return _KnitAnnotator(self)
 
-    def check(self, progress_bar=None):
+    def check(self, progress_bar=None, keys=None):
         """See VersionedFiles.check()."""
+        if keys is None:
+            return self._logical_check()
+        else:
+            # At the moment, check does not extra work over get_record_stream
+            return self.get_record_stream(keys, 'unordered', True)
+
+    def _logical_check(self):
         # This doesn't actually test extraction of everything, but that will
         # impact 'bzr check' substantially, and needs to be integrated with
         # care. However, it does check for the obvious problem of a delta with

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2009-07-30 04:27:05 +0000
+++ bzrlib/remote.py	2009-08-05 02:30:59 +0000
@@ -1470,9 +1470,10 @@
         return self._real_repository.get_revision_reconcile(revision_id)
 
     @needs_read_lock
-    def check(self, revision_ids=None):
+    def check(self, revision_ids=None, callback_refs=None, check_repo=True):
         self._ensure_real()
-        return self._real_repository.check(revision_ids=revision_ids)
+        return self._real_repository.check(revision_ids=revision_ids,
+            callback_refs=callback_refs, check_repo=check_repo)
 
     def copy_content_into(self, destination, revision_id=None):
         self._ensure_real()
@@ -1618,9 +1619,10 @@
         self._ensure_real()
         return self._real_repository.revision_graph_can_have_wrong_parents()
 
-    def _find_inconsistent_revision_parents(self):
+    def _find_inconsistent_revision_parents(self, revisions_iterator=None):
         self._ensure_real()
-        return self._real_repository._find_inconsistent_revision_parents()
+        return self._real_repository._find_inconsistent_revision_parents(
+            revisions_iterator)
 
     def _check_for_inconsistent_revision_parents(self):
         self._ensure_real()

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2009-07-14 17:33:13 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2009-08-04 04:36:34 +0000
@@ -766,6 +766,10 @@
             if basis_tree is not None:
                 basis_tree.unlock()
 
+    def deserialise_inventory(self, revision_id, bytes):
+        return inventory.CHKInventory.deserialise(self.chk_bytes, bytes,
+            (revision_id,))
+
     def _iter_inventories(self, revision_ids):
         """Iterate over many inventory objects."""
         keys = [(revision_id,) for revision_id in revision_ids]

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2009-06-10 03:56:49 +0000
+++ bzrlib/repofmt/knitrepo.py	2009-06-16 05:29:42 +0000
@@ -229,24 +229,29 @@
     def _make_parents_provider(self):
         return _KnitsParentsProvider(self.revisions)
 
-    def _find_inconsistent_revision_parents(self):
+    def _find_inconsistent_revision_parents(self, revisions_iterator=None):
         """Find revisions with different parent lists in the revision object
         and in the index graph.
 
+        :param revisions_iterator: None, or an iterator of (revid,
+            Revision-or-None). This iterator controls the revisions checked.
         :returns: an iterator yielding tuples of (revison-id, parents-in-index,
             parents-in-revision).
         """
         if not self.is_locked():
             raise AssertionError()
         vf = self.revisions
-        for index_version in vf.keys():
-            parent_map = vf.get_parent_map([index_version])
+        if revisions_iterator is None:
+            revisions_iterator = self._iter_revisions(None)
+        for revid, revision in revisions_iterator:
+            if revision is None:
+                pass
+            parent_map = vf.get_parent_map([(revid,)])
             parents_according_to_index = tuple(parent[-1] for parent in
-                parent_map[index_version])
-            revision = self.get_revision(index_version[-1])
+                parent_map[(revid,)])
             parents_according_to_revision = tuple(revision.parent_ids)
             if parents_according_to_index != parents_according_to_revision:
-                yield (index_version[-1], parents_according_to_index,
+                yield (revid, parents_according_to_index,
                     parents_according_to_revision)
 
     def _check_for_inconsistent_revision_parents(self):

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2009-07-01 10:42:14 +0000
+++ bzrlib/repofmt/pack_repo.py	2009-08-05 01:05:58 +0000
@@ -2215,52 +2215,6 @@
         self.revisions._index._key_dependencies.refs.clear()
         self._pack_collection._abort_write_group()
 
-    def _find_inconsistent_revision_parents(self):
-        """Find revisions with incorrectly cached parents.
-
-        :returns: an iterator yielding tuples of (revison-id, parents-in-index,
-            parents-in-revision).
-        """
-        if not self.is_locked():
-            raise errors.ObjectNotLocked(self)
-        pb = ui.ui_factory.nested_progress_bar()
-        result = []
-        try:
-            revision_nodes = self._pack_collection.revision_index \
-                .combined_index.iter_all_entries()
-            index_positions = []
-            # Get the cached index values for all revisions, and also the
-            # location in each index of the revision text so we can perform
-            # linear IO.
-            for index, key, value, refs in revision_nodes:
-                node = (index, key, value, refs)
-                index_memo = self.revisions._index._node_to_position(node)
-                if index_memo[0] != index:
-                    raise AssertionError('%r != %r' % (index_memo[0], index))
-                index_positions.append((index_memo, key[0],
-                                       tuple(parent[0] for parent in refs[0])))
-                pb.update("Reading revision index", 0, 0)
-            index_positions.sort()
-            batch_size = 1000
-            pb.update("Checking cached revision graph", 0,
-                      len(index_positions))
-            for offset in xrange(0, len(index_positions), 1000):
-                pb.update("Checking cached revision graph", offset)
-                to_query = index_positions[offset:offset + batch_size]
-                if not to_query:
-                    break
-                rev_ids = [item[1] for item in to_query]
-                revs = self.get_revisions(rev_ids)
-                for revision, item in zip(revs, to_query):
-                    index_parents = item[2]
-                    rev_parents = tuple(revision.parent_ids)
-                    if index_parents != rev_parents:
-                        result.append((revision.revision_id, index_parents,
-                                       rev_parents))
-        finally:
-            pb.finished()
-        return result
-
     def _get_source(self, to_format):
         if to_format.network_name() == self._format.network_name():
             return KnitPackStreamSource(self, to_format)

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2009-08-04 16:20:05 +0000
+++ bzrlib/repository.py	2009-08-06 02:23:37 +0000
@@ -1146,6 +1146,119 @@
         # The old API returned a list, should this actually be a set?
         return parent_map.keys()
 
+    def _check_inventories(self, checker):
+        """Check the inventories found from the revision scan.
+        
+        This is responsible for verifying the sha1 of inventories and
+        creating a pending_keys set that covers data referenced by inventories.
+        """
+        bar = ui.ui_factory.nested_progress_bar()
+        try:
+            self._do_check_inventories(checker, bar)
+        finally:
+            bar.finished()
+
+    def _do_check_inventories(self, checker, bar):
+        """Helper for _check_inventories."""
+        revno = 0
+        keys = {'chk_bytes':set(), 'inventories':set(), 'texts':set()}
+        kinds = ['chk_bytes', 'texts']
+        count = len(checker.pending_keys)
+        bar.update("inventories", 0, 2)
+        current_keys = checker.pending_keys
+        checker.pending_keys = {}
+        # Accumulate current checks.
+        for key in current_keys:
+            if key[0] != 'inventories' and key[0] not in kinds:
+                checker._report_items.append('unknown key type %r' % (key,))
+            keys[key[0]].add(key[1:])
+        if keys['inventories']:
+            # NB: output order *should* be roughly sorted - topo or
+            # inverse topo depending on repository - either way decent
+            # to just delta against. However, pre-CHK formats didn't
+            # try to optimise inventory layout on disk. As such the
+            # pre-CHK code path does not use inventory deltas.
+            last_object = None
+            for record in self.inventories.check(keys=keys['inventories']):
+                if record.storage_kind == 'absent':
+                    checker._report_items.append(
+                        'Missing inventory {%s}' % (record.key,))
+                else:
+                    last_object = self._check_record('inventories', record,
+                        checker, last_object,
+                        current_keys[('inventories',) + record.key])
+            del keys['inventories']
+        else:
+            return
+        bar.update("texts", 1)
+        while (checker.pending_keys or keys['chk_bytes']
+            or keys['texts']):
+            # Something to check.
+            current_keys = checker.pending_keys
+            checker.pending_keys = {}
+            # Accumulate current checks.
+            for key in current_keys:
+                if key[0] not in kinds:
+                    checker._report_items.append('unknown key type %r' % (key,))
+                keys[key[0]].add(key[1:])
+            # Check the outermost kind only - inventories || chk_bytes || texts
+            for kind in kinds:
+                if keys[kind]:
+                    last_object = None
+                    for record in getattr(self, kind).check(keys=keys[kind]):
+                        if record.storage_kind == 'absent':
+                            checker._report_items.append(
+                                'Missing inventory {%s}' % (record.key,))
+                        else:
+                            last_object = self._check_record(kind, record,
+                                checker, last_object, current_keys[(kind,) + record.key])
+                    keys[kind] = set()
+                    break
+
+    def _check_record(self, kind, record, checker, last_object, item_data):
+        """Check a single text from this repository."""
+        if kind == 'inventories':
+            rev_id = record.key[0]
+            inv = self.deserialise_inventory(rev_id,
+                record.get_bytes_as('fulltext'))
+            if last_object is not None:
+                delta = inv._make_delta(last_object)
+                for old_path, path, file_id, ie in delta:
+                    if ie is None:
+                        continue
+                    ie.check(checker, rev_id, inv)
+            else:
+                for path, ie in inv.iter_entries():
+                    ie.check(checker, rev_id, inv)
+            if self._format.fast_deltas:
+                return inv
+        elif kind == 'chk_bytes':
+            # No code written to check chk_bytes for this repo format.
+            checker._report_items.append(
+                'unsupported key type chk_bytes for %s' % (record.key,))
+        elif kind == 'texts':
+            self._check_text(record, checker, item_data)
+        else:
+            checker._report_items.append(
+                'unknown key type %s for %s' % (kind, record.key))
+
+    def _check_text(self, record, checker, item_data):
+        """Check a single text."""
+        # Check it is extractable.
+        # TODO: check length.
+        if record.storage_kind == 'chunked':
+            chunks = record.get_bytes_as(record.storage_kind)
+            sha1 = osutils.sha_strings(chunks)
+            length = sum(map(len, chunks))
+        else:
+            content = record.get_bytes_as('fulltext')
+            sha1 = osutils.sha_string(content)
+            length = len(content)
+        if item_data and sha1 != item_data[1]:
+            checker._report_items.append(
+                'sha1 mismatch: %s has sha1 %s expected %s referenced by %s' %
+                (record.key, sha1, item_data[1], item_data[2]))
+
     @staticmethod
     def create(a_bzrdir):
         """Construct the current default format repository in a_bzrdir."""
@@ -1714,25 +1827,49 @@
 
     @needs_read_lock
     def get_revisions(self, revision_ids):
-        """Get many revisions at once."""
+        """Get many revisions at once.
+        
+        Repositories that need to check data on every revision read should 
+        subclass this method.
+        """
         return self._get_revisions(revision_ids)
 
     @needs_read_lock
     def _get_revisions(self, revision_ids):
         """Core work logic to get many revisions without sanity checks."""
-        for rev_id in revision_ids:
-            if not rev_id or not isinstance(rev_id, basestring):
-                raise errors.InvalidRevisionId(revision_id=rev_id, branch=self)
+        revs = {}
+        for revid, rev in self._iter_revisions(revision_ids):
+            if rev is None:
+                raise errors.NoSuchRevision(self, revid)
+            revs[revid] = rev
+        return [revs[revid] for revid in revision_ids]
+
+    def _iter_revisions(self, revision_ids):
+        """Iterate over revision objects.
+
+        :param revision_ids: An iterable of revisions to examine. None may be
+            passed to request all revisions known to the repository. Note that
+            not all repositories can find unreferenced revisions; for those
+            repositories only referenced ones will be returned.
+        :return: An iterator of (revid, revision) tuples. Absent revisions (
+            those asked for but not available) are returned as (revid, None).
+        """
+        if revision_ids is None:
+            revision_ids = self.all_revision_ids()
+        else:
+            for rev_id in revision_ids:
+                if not rev_id or not isinstance(rev_id, basestring):
+                    raise errors.InvalidRevisionId(revision_id=rev_id, branch=self)
         keys = [(key,) for key in revision_ids]
         stream = self.revisions.get_record_stream(keys, 'unordered', True)
-        revs = {}
         for record in stream:
+            revid = record.key[0]
             if record.storage_kind == 'absent':
-                raise errors.NoSuchRevision(self, record.key[0])
-            text = record.get_bytes_as('fulltext')
-            rev = self._serializer.read_revision_from_string(text)
-            revs[record.key[0]] = rev
-        return [revs[revid] for revid in revision_ids]
+                yield (revid, None)
+            else:
+                text = record.get_bytes_as('fulltext')
+                rev = self._serializer.read_revision_from_string(text)
+                yield (revid, rev)
 
     @needs_read_lock
     def get_revision_xml(self, revision_id):
@@ -2093,8 +2230,7 @@
                 batch_size]
             if not to_query:
                 break
-            for rev_tree in self.revision_trees(to_query):
-                revision_id = rev_tree.get_revision_id()
+            for revision_id in to_query:
                 parent_ids = ancestors[revision_id]
                 for text_key in revision_keys[revision_id]:
                     pb.update("Calculating text parents", processed_texts)
@@ -2496,7 +2632,8 @@
                 [parents_provider, other_repository._make_parents_provider()])
         return graph.Graph(parents_provider)
 
-    def _get_versioned_file_checker(self, text_key_references=None):
+    def _get_versioned_file_checker(self, text_key_references=None,
+        ancestors=None):
         """Return an object suitable for checking versioned files.
         
         :param text_key_references: if non-None, an already built
@@ -2504,9 +2641,12 @@
             to whether they were referred to by the inventory of the
             revision_id that they contain. If None, this will be
             calculated.
+        :param ancestors: Optional result from
+            self.get_graph().get_parent_map(self.all_revision_ids()) if already
+            available.
         """
         return _VersionedFileChecker(self,
-            text_key_references=text_key_references)
+            text_key_references=text_key_references, ancestors=ancestors)
 
     def revision_ids_to_search_result(self, result_set):
         """Convert a set of revision ids to a graph SearchResult."""
@@ -2562,19 +2702,25 @@
         return record.get_bytes_as('fulltext')
 
     @needs_read_lock
-    def check(self, revision_ids=None):
+    def check(self, revision_ids=None, callback_refs=None, check_repo=True):
         """Check consistency of all history of given revision_ids.
 
         Different repository implementations should override _check().
 
         :param revision_ids: A non-empty list of revision_ids whose ancestry
              will be checked.  Typically the last revision_id of a branch.
+        :param callback_refs: A dict of check-refs to resolve and callback
+            the check/_check method on the items listed as wanting the ref.
+            see bzrlib.check.
+        :param check_repo: If False do not check the repository contents, just 
+            calculate the data callback_refs requires and call them back.
         """
-        return self._check(revision_ids)
+        return self._check(revision_ids, callback_refs=callback_refs,
+            check_repo=check_repo)
 
-    def _check(self, revision_ids):
-        result = check.Check(self)
-        result.check()
+    def _check(self, revision_ids, callback_refs, check_repo):
+        result = check.Check(self, check_repo=check_repo)
+        result.check(callback_refs)
         return result
 
     def _warn_if_deprecated(self):
@@ -3873,10 +4019,10 @@
 
 class _VersionedFileChecker(object):
 
-    def __init__(self, repository, text_key_references=None):
+    def __init__(self, repository, text_key_references=None, ancestors=None):
         self.repository = repository
         self.text_index = self.repository._generate_text_key_index(
-            text_key_references=text_key_references)
+            text_key_references=text_key_references, ancestors=ancestors)
 
     def calculate_file_version_parents(self, text_key):
         """Calculate the correct parents for a file version according to
@@ -3900,6 +4046,18 @@
             revision_id) tuples for versions that are present in this versioned
             file, but not used by the corresponding inventory.
         """
+        local_progress = None
+        if progress_bar is None:
+            local_progress = ui.ui_factory.nested_progress_bar()
+            progress_bar = local_progress
+        try:
+            return self._check_file_version_parents(texts, progress_bar)
+        finally:
+            if local_progress:
+                local_progress.finished()
+
+    def _check_file_version_parents(self, texts, progress_bar):
+        """See check_file_version_parents."""
         wrong_parents = {}
         self.file_ids = set([file_id for file_id, _ in
             self.text_index.iterkeys()])
@@ -3914,8 +4072,7 @@
         text_keys = self.repository.texts.keys()
         unused_keys = frozenset(text_keys) - set(self.text_index)
         for num, key in enumerate(self.text_index.iterkeys()):
-            if progress_bar is not None:
-                progress_bar.update('checking text graph', num, n_versions)
+            progress_bar.update('checking text graph', num, n_versions)
             correct_parents = self.calculate_file_version_parents(key)
             try:
                 knit_parents = parent_map[key]

=== modified file 'bzrlib/smart/medium.py'
--- bzrlib/smart/medium.py	2009-07-20 03:23:31 +0000
+++ bzrlib/smart/medium.py	2009-08-04 04:36:34 +0000
@@ -37,7 +37,6 @@
 from bzrlib import (
     debug,
     errors,
-    osutils,
     symbol_versioning,
     trace,
     ui,
@@ -46,7 +45,8 @@
 from bzrlib.smart import client, protocol, request, vfs
 from bzrlib.transport import ssh
 """)
-
+#usually already imported, and getting IllegalScoperReplacer on it here.
+from bzrlib import osutils
 
 # We must not read any more than 64k at a time so we don't risk "no buffer
 # space available" errors on some platforms.  Windows in particular is likely

=== modified file 'bzrlib/tests/blackbox/test_check.py'
--- bzrlib/tests/blackbox/test_check.py	2009-08-03 04:40:07 +0000
+++ bzrlib/tests/blackbox/test_check.py	2009-08-04 04:36:34 +0000
@@ -34,24 +34,22 @@
         tree = self.make_branch_and_tree('.')
         tree.commit('hallelujah')
         out, err = self.run_bzr('check')
+        self.assertContainsRe(err, r"Checking working tree at '.*'\.\n")
+        self.assertContainsRe(err, r"Checking repository at '.*'\.\n")
         # the root directory may be in the texts for rich root formats
-        self.assertContainsRe(err, r"^Checking working tree at '.*'\.\n"
-                                   r"Checking repository at '.*'\.\n"
-                                   r"checked repository.*\n"
+        self.assertContainsRe(err, r"checked repository.*\n"
                                    r"     1 revisions\n"
                                    r"     [01] file-ids\n"
-                                   r"     0 unique file texts\n"
-                                   r"     0 repeated file texts\n"
-                                   r"     0 unreferenced text versions\n"
-                                   r"Checking branch at '.*'\.\n"
-                                   r"checked branch.*\n$")
+                                   )
+        self.assertContainsRe(err, r"Checking branch at '.*'\.\n")
+        self.assertContainsRe(err, r"checked branch.*")
 
     def test_check_branch(self):
         tree = self.make_branch_and_tree('.')
         tree.commit('foo')
         out, err = self.run_bzr('check --branch')
         self.assertContainsRe(err, r"^Checking branch at '.*'\.\n"
-                                   r"checked branch.*\n$")
+                                   r"checked branch.*")
 
     def test_check_repository(self):
         tree = self.make_branch_and_tree('.')
@@ -61,9 +59,7 @@
                                    r"checked repository.*\n"
                                    r"     1 revisions\n"
                                    r"     [01] file-ids\n"
-                                   r"     0 unique file texts\n"
-                                   r"     0 repeated file texts\n"
-                                   r"     0 unreferenced text versions$")
+                                   )
 
     def test_check_tree(self):
         tree = self.make_branch_and_tree('.')
@@ -77,7 +73,7 @@
         out, err = self.run_bzr('check --tree --branch')
         self.assertContainsRe(err, r"^Checking working tree at '.*'\.\n"
                                    r"Checking branch at '.*'\.\n"
-                                   r"checked branch.*\n$")
+                                   r"checked branch.*")
 
     def test_check_missing_tree(self):
         branch = self.make_branch('.')
@@ -88,9 +84,9 @@
         branch = self.make_branch('.')
         out, err = self.run_bzr('check --tree --branch')
         self.assertContainsRe(err,
-            r"^No working tree found at specified location\.\n"
             r"Checking branch at '.*'\.\n"
-            r"checked branch.*\n$")
+            r"No working tree found at specified location\.\n"
+            r"checked branch.*")
 
     def test_check_missing_branch_in_shared_repo(self):
         self.make_repository('shared', shared=True)

=== modified file 'bzrlib/tests/per_branch/test_check.py'
--- bzrlib/tests/per_branch/test_check.py	2009-07-10 05:49:34 +0000
+++ bzrlib/tests/per_branch/test_check.py	2009-08-04 04:36:34 +0000
@@ -16,7 +16,9 @@
 
 """Tests for branch implementations - test check() functionality"""
 
-from bzrlib import errors
+from StringIO import StringIO
+
+from bzrlib import errors, tests, ui
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
 
@@ -54,24 +56,57 @@
             # with set_last_revision_info
             tree.branch.set_last_revision_info(3, r5)
 
-        e = self.assertRaises(errors.BzrCheckError,
-                              tree.branch.check)
-        self.assertEqual('Internal check failed:'
-                         ' revno does not match len(mainline) 3 != 5', str(e))
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        refs = self.make_refs(tree.branch)
+        result = tree.branch.check(refs)
+        ui.ui_factory = tests.TestUIFactory(stdout=StringIO())
+        result.report_results(True)
+        self.assertContainsRe('revno does not match len',
+            ui.ui_factory.stdout.getvalue())
 
     def test_check_branch_report_results(self):
         """Checking a branch produces results which can be printed"""
         branch = self.make_branch('.')
-        result = branch.check()
+        branch.lock_read()
+        self.addCleanup(branch.unlock)
+        result = branch.check(self.make_refs(branch))
         # reports results through logging
         result.report_results(verbose=True)
         result.report_results(verbose=False)
 
-    def test_check_detects_ghosts_in_mainline(self):
-        tree = self.make_branch_and_tree('test')
-        tree.set_parent_ids(['thisisaghost'], allow_leftmost_as_ghost=True)
-        r1 = tree.commit('one')
-        r2 = tree.commit('two')
-        result = tree.branch.check()
-        self.assertEquals(True, result.ghosts_in_mainline)
+    def test__get_check_refs(self):
+        tree = self.make_branch_and_tree('.')
+        revid = tree.commit('foo')
+        self.assertEqual(
+            set([('revision-existence', revid), ('lefthand-distance', revid)]),
+            set(tree.branch._get_check_refs()))
 
+    def make_refs(self, branch):
+        needed_refs = branch._get_check_refs()
+        refs = {}
+        distances = set()
+        existences = set()
+        for ref in needed_refs:
+            kind, value = ref
+            if kind == 'lefthand-distance':
+                distances.add(value)
+            elif kind == 'revision-existence':
+                existences.add(value)
+            else:
+                raise AssertionError(
+                    'unknown ref kind for ref %s' % ref)
+        node_distances = branch.repository.get_graph().find_lefthand_distances(
+            distances)
+        for key, distance in node_distances.iteritems():
+            refs[('lefthand-distance', key)] = distance
+            if key in existences and distance > 0:
+                refs[('revision-existence', key)] = True
+                existences.remove(key)
+        parent_map = branch.repository.get_graph().get_parent_map(existences)
+        for key in parent_map:
+            refs[('revision-existence', key)] = True
+            existences.remove(key)
+        for key in existences:
+            refs[('revision-existence', key)] = False
+        return refs

=== modified file 'bzrlib/tests/per_branch/test_push.py'
--- bzrlib/tests/per_branch/test_push.py	2009-07-10 05:49:34 +0000
+++ bzrlib/tests/per_branch/test_push.py	2009-08-05 02:30:59 +0000
@@ -23,6 +23,7 @@
     branch,
     builtins,
     bzrdir,
+    check,
     debug,
     errors,
     push,
@@ -224,7 +225,7 @@
         # fulltext record for f-id @ rev-1, then this will fail.
         remote_branch = Branch.open(self.get_url('remote'))
         trunk.push(remote_branch)
-        remote_branch.check()
+        check.check_dwim(remote_branch.base, False, True, True)
 
     def test_no_get_parent_map_after_insert_stream(self):
         # Effort test for bug 331823

=== modified file 'bzrlib/tests/per_branch/test_stacking.py'
--- bzrlib/tests/per_branch/test_stacking.py	2009-07-24 03:15:56 +0000
+++ bzrlib/tests/per_branch/test_stacking.py	2009-08-05 02:30:59 +0000
@@ -19,6 +19,7 @@
 from bzrlib import (
     branch,
     bzrdir,
+    check,
     errors,
     )
 from bzrlib.revision import NULL_REVISION
@@ -360,7 +361,7 @@
             self.build_tree_contents([('stacked/a', ''.join(text_lines))])
             stacked_tree.commit('commit %d' % i)
         stacked_tree.branch.repository.pack()
-        stacked_tree.branch.check()
+        check.check_dwim(stacked_tree.branch.base, False, True, True)
 
     def test_pull_delta_when_stacked(self):
         if not self.branch_format.supports_stacking():
@@ -384,7 +385,7 @@
         # bug 252821 caused a RevisionNotPresent here...
         stacked_tree.pull(other_tree.branch)
         stacked_tree.branch.repository.pack()
-        stacked_tree.branch.check()
+        check.check_dwim(stacked_tree.branch.base, False, True, True)
         self.check_lines_added_or_present(stacked_tree.branch, stacked_revid)
 
     def test_fetch_revisions_with_file_changes(self):

=== modified file 'bzrlib/tests/per_bzrdir/test_bzrdir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	2009-07-17 10:38:41 +0000
+++ bzrlib/tests/per_bzrdir/test_bzrdir.py	2009-08-06 00:48:50 +0000
@@ -26,6 +26,7 @@
 import bzrlib.branch
 from bzrlib import (
     bzrdir,
+    check,
     errors,
     lockdir,
     osutils,
@@ -38,7 +39,6 @@
     workingtree,
     )
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
-from bzrlib.check import check_branch
 from bzrlib.errors import (FileExists,
                            NoSuchRevision,
                            NoSuchFile,
@@ -1738,8 +1738,7 @@
             finally:
                 pb.finished()
             # and it should pass 'check' now.
-            check_branch(bzrdir.BzrDir.open(self.get_url('.')).open_branch(),
-                         False)
+            check.check_dwim(self.get_url('.'), False, True, True)
 
     def test_format_description(self):
         dir = self.make_bzrdir('.')

=== modified file 'bzrlib/tests/per_interbranch/test_push.py'
--- bzrlib/tests/per_interbranch/test_push.py	2009-07-10 05:49:34 +0000
+++ bzrlib/tests/per_interbranch/test_push.py	2009-08-05 02:30:59 +0000
@@ -23,6 +23,7 @@
     branch,
     builtins,
     bzrdir,
+    check,
     debug,
     errors,
     push,
@@ -230,7 +231,7 @@
         # fulltext record for f-id @ rev-1, then this will fail.
         remote_branch = Branch.open(self.get_url('remote'))
         trunk.push(remote_branch)
-        remote_branch.check()
+        check.check_dwim(remote_branch.base, False, True, True)
 
     def test_no_get_parent_map_after_insert_stream(self):
         # Effort test for bug 331823

=== modified file 'bzrlib/tests/per_repository/test_check.py'
--- bzrlib/tests/per_repository/test_check.py	2009-04-09 20:23:07 +0000
+++ bzrlib/tests/per_repository/test_check.py	2009-05-12 05:34:15 +0000
@@ -36,18 +36,14 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['foo'])
         tree.smart_add(['.'])
-        tree.commit('1')
+        revid1 = tree.commit('1')
         self.build_tree(['bar'])
         tree.smart_add(['.'])
-        tree.commit('2')
-        # XXX: check requires a non-empty revision IDs list, but it ignores the
-        # contents of it!
-        check_object = tree.branch.repository.check(['ignored'])
+        revid2 = tree.commit('2')
+        check_object = tree.branch.repository.check([revid1, revid2])
         check_object.report_results(verbose=True)
         log = self._get_log(keep_log_file=True)
-        self.assertContainsRe(
-            log,
-            "0 unreferenced text versions")
+        self.assertContainsRe(log, "0 unreferenced text versions")
 
 
 class TestFindInconsistentRevisionParents(TestCaseWithBrokenRevisionIndex):
@@ -100,3 +96,32 @@
             "revision-id has wrong parents in index: "
             r"\('incorrect-parent',\) should be \(\)")
 
+
+class TestCallbacks(TestCaseWithRepository):
+
+    def test_callback_tree_and_branch(self):
+        # use a real tree to get actual refs that will work
+        tree = self.make_branch_and_tree('foo')
+        revid = tree.commit('foo')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        needed_refs = {}
+        for ref in tree._get_check_refs():
+            needed_refs.setdefault(ref, []).append(tree)
+        for ref in tree.branch._get_check_refs():
+            needed_refs.setdefault(ref, []).append(tree.branch)
+        self.tree_check = tree._check
+        self.branch_check = tree.branch.check
+        tree._check = self.tree_callback
+        tree.branch.check = self.branch_callback
+        self.callbacks = []
+        tree.branch.repository.check([revid], callback_refs=needed_refs)
+        self.assertNotEqual([], self.callbacks)
+
+    def tree_callback(self, refs):
+        self.callbacks.append(('tree', refs))
+        return self.tree_check(refs)
+
+    def branch_callback(self, refs):
+        self.callbacks.append(('branch', refs))
+        return self.branch_check(refs)

=== modified file 'bzrlib/tests/per_workingtree/__init__.py'
--- bzrlib/tests/per_workingtree/__init__.py	2009-07-10 12:40:36 +0000
+++ bzrlib/tests/per_workingtree/__init__.py	2009-08-04 04:36:34 +0000
@@ -66,7 +66,7 @@
 
 
 def load_tests(standard_tests, module, loader):
-    per_wt_mod_names = [
+    test_names = [
         'add_reference',
         'add',
         'annotate_iter',
@@ -74,6 +74,7 @@
         'basis_tree',
         'break_lock',
         'changes_from',
+        'check',
         'content_filters',
         'commit',
         'eol_conversion',
@@ -106,6 +107,9 @@
         'walkdirs',
         'workingtree',
         ]
+    test_workingtree_implementations = [
+        'bzrlib.tests.per_workingtree.test_' + name for
+        name in test_names]
 
     scenarios = make_scenarios(
         tests.default_transport,
@@ -117,7 +121,5 @@
 
     # add the tests for the sub modules
     return tests.multiply_tests(
-        loader.loadTestsFromModuleNames(
-        ['bzrlib.tests.per_workingtree.test_' + name
-         for name in per_wt_mod_names]),
+        loader.loadTestsFromModuleNames(test_workingtree_implementations),
         scenarios, standard_tests)

=== added file 'bzrlib/tests/per_workingtree/test_check.py'
--- bzrlib/tests/per_workingtree/test_check.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_workingtree/test_check.py	2009-08-04 04:47:35 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for checking of trees."""
+
+from bzrlib import (
+    tests,
+    )
+from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+
+
+class TestCheck(TestCaseWithWorkingTree):
+
+    def test__get_check_refs_new(self):
+        tree = self.make_branch_and_tree('tree')
+        self.assertEqual(set([('trees', 'null:')]),
+            set(tree._get_check_refs()))
+
+    def test__get_check_refs_basis(self):
+        # with a basis, all current bzr trees cache it and so need the
+        # inventory to cross-check.
+        tree = self.make_branch_and_tree('tree')
+        revid = tree.commit('first post')
+        self.assertEqual(set([('trees', revid)]),
+            set(tree._get_check_refs()))
+
+    def test__check_with_refs(self):
+        # _check can be called with a dict of the things required.
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        revid = tree.commit('first post')
+        needed_refs = tree._get_check_refs()
+        repo = tree.branch.repository
+        for ref in needed_refs:
+            kind, revid = ref
+            refs = {}
+            if kind == 'trees':
+                refs[ref] = repo.revision_tree(revid)
+            else:
+                self.fail('unknown ref kind')
+        tree._check(refs)

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	2009-06-17 19:08:25 +0000
+++ bzrlib/tests/test_commit_merge.py	2009-08-05 02:30:59 +0000
@@ -18,14 +18,14 @@
 import os
 import shutil
 
+from bzrlib import check, osutils
+from bzrlib.branch import Branch
+from bzrlib.errors import PointlessCommit, BzrError
 from bzrlib.tests import (
     SymlinkFeature,
     TestCaseWithTransport,
     )
-from bzrlib.branch import Branch
-from bzrlib.errors import PointlessCommit, BzrError
 from bzrlib.tests.test_revision import make_branches
-from bzrlib import osutils
 
 
 class TestCommitMerge(TestCaseWithTransport):
@@ -96,10 +96,8 @@
         self.assertEquals(inv['ecks-id'].revision, 'x@u-0-1')
         self.assertEquals(inv['why-id'].revision, 'y@u-0-1')
 
-        bx.check()
-        by.check()
-        bx.repository.check([bx.last_revision()])
-        by.repository.check([by.last_revision()])
+        check.check_dwim(bx.base, False, True, True)
+        check.check_dwim(by.base, False, True, True)
 
     def test_merge_with_symlink(self):
         self.requireFeature(SymlinkFeature)

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2009-06-10 18:58:02 +0000
+++ bzrlib/tests/test_graph.py	2009-08-04 04:36:34 +0000
@@ -526,6 +526,19 @@
         graph = self.make_graph(history_shortcut)
         self.assertEqual(set(['rev2b']), graph.find_lca('rev3a', 'rev3b'))
 
+    def test_lefthand_distance_smoke(self):
+        """A simple does it work test for graph.lefthand_distance(keys)."""
+        graph = self.make_graph(history_shortcut)
+        distance_graph = graph.find_lefthand_distances(['rev3b', 'rev2a'])
+        self.assertEqual({'rev2a': 2, 'rev3b': 3}, distance_graph)
+
+    def test_lefthand_distance_ghosts(self):
+        """A simple does it work test for graph.lefthand_distance(keys)."""
+        nodes = {'nonghost':[NULL_REVISION], 'toghost':['ghost']}
+        graph = self.make_graph(nodes)
+        distance_graph = graph.find_lefthand_distances(['nonghost', 'toghost'])
+        self.assertEqual({'nonghost': 1, 'toghost': -1}, distance_graph)
+
     def test_recursive_unique_lca(self):
         """Test finding a unique least common ancestor.
 

=== modified file 'bzrlib/tests/test_upgrade_stacked.py'
--- bzrlib/tests/test_upgrade_stacked.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_upgrade_stacked.py	2009-08-05 02:30:59 +0000
@@ -19,6 +19,7 @@
 
 from bzrlib import (
     bzrdir,
+    check,
     errors,
     tests,
     )
@@ -56,13 +57,14 @@
             self.assertRaises(errors.IncompatibleRepositories,
                 stacked.open_branch)
         else:
-            stacked.open_branch().check()
+            check.check_dwim('stacked', False, True, True)
         stacked = bzrdir.BzrDir.open('stacked')
         # but we can upgrade the stacked repository
         upgrade('stacked', new_format)
-        # and now it's ok
+        # and now it opens ok
         stacked = bzrdir.BzrDir.open('stacked')
-        stacked.open_branch().check()
+        # And passes check.
+        check.check_dwim('stacked', False, True, True)
 
 
 def load_tests(basic_tests, module, loader):

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2009-07-06 20:21:34 +0000
+++ bzrlib/tests/test_versionedfile.py	2009-08-04 04:36:34 +0000
@@ -30,6 +30,7 @@
     knit as _mod_knit,
     osutils,
     progress,
+    ui,
     )
 from bzrlib.errors import (
                            RevisionNotPresent,
@@ -1557,41 +1558,27 @@
         self.assertRaises(RevisionNotPresent,
             files.annotate, prefix + ('missing-key',))
 
-    def test_get_annotator(self):
+    def test_check_no_parameters(self):
+        files = self.get_versionedfiles()
+
+    def test_check_progressbar_parameter(self):
+        """A progress bar can be supplied because check can be a generator."""
+        pb = ui.ui_factory.nested_progress_bar()
+        self.addCleanup(pb.finished)
+        files = self.get_versionedfiles()
+        files.check(progress_bar=pb)
+
+    def test_check_with_keys_becomes_generator(self):
         files = self.get_versionedfiles()
         self.get_diamond_files(files)
-        origin_key = self.get_simple_key('origin')
-        base_key = self.get_simple_key('base')
-        left_key = self.get_simple_key('left')
-        right_key = self.get_simple_key('right')
-        merged_key = self.get_simple_key('merged')
-        # annotator = files.get_annotator()
-        # introduced full text
-        origins, lines = files.get_annotator().annotate(origin_key)
-        self.assertEqual([(origin_key,)], origins)
-        self.assertEqual(['origin\n'], lines)
-        # a delta
-        origins, lines = files.get_annotator().annotate(base_key)
-        self.assertEqual([(base_key,)], origins)
-        # a merge
-        origins, lines = files.get_annotator().annotate(merged_key)
-        if self.graph:
-            self.assertEqual([
-                (base_key,),
-                (left_key,),
-                (right_key,),
-                (merged_key,),
-                ], origins)
-        else:
-            # Without a graph everything is new.
-            self.assertEqual([
-                (merged_key,),
-                (merged_key,),
-                (merged_key,),
-                (merged_key,),
-                ], origins)
-        self.assertRaises(RevisionNotPresent,
-            files.get_annotator().annotate, self.get_simple_key('missing-key'))
+        keys = files.keys()
+        entries = files.check(keys=keys)
+        seen = set()
+        # Texts output should be fulltexts.
+        self.capture_stream(files, entries, seen.add,
+            files.get_parent_map(keys), require_fulltext=True)
+        # All texts should be output.
+        self.assertEqual(set(keys), seen)
 
     def test_construct(self):
         """Each parameterised test can be constructed on a transport."""
@@ -1766,7 +1753,8 @@
              'knit-delta-closure', 'knit-delta-closure-ref',
              'groupcompress-block', 'groupcompress-block-ref'])
 
-    def capture_stream(self, f, entries, on_seen, parents):
+    def capture_stream(self, f, entries, on_seen, parents,
+        require_fulltext=False):
         """Capture a stream for testing."""
         for factory in entries:
             on_seen(factory.key)
@@ -1777,6 +1765,8 @@
             self.assertEqual(parents[factory.key], factory.parents)
             self.assertIsInstance(factory.get_bytes_as(factory.storage_kind),
                 str)
+            if require_fulltext:
+                factory.get_bytes_as('fulltext')
 
     def test_get_record_stream_interface(self):
         """each item in a stream has to provide a regular interface."""
@@ -2157,6 +2147,42 @@
         else:
             return None
 
+    def test_get_annotator(self):
+        files = self.get_versionedfiles()
+        self.get_diamond_files(files)
+        origin_key = self.get_simple_key('origin')
+        base_key = self.get_simple_key('base')
+        left_key = self.get_simple_key('left')
+        right_key = self.get_simple_key('right')
+        merged_key = self.get_simple_key('merged')
+        # annotator = files.get_annotator()
+        # introduced full text
+        origins, lines = files.get_annotator().annotate(origin_key)
+        self.assertEqual([(origin_key,)], origins)
+        self.assertEqual(['origin\n'], lines)
+        # a delta
+        origins, lines = files.get_annotator().annotate(base_key)
+        self.assertEqual([(base_key,)], origins)
+        # a merge
+        origins, lines = files.get_annotator().annotate(merged_key)
+        if self.graph:
+            self.assertEqual([
+                (base_key,),
+                (left_key,),
+                (right_key,),
+                (merged_key,),
+                ], origins)
+        else:
+            # Without a graph everything is new.
+            self.assertEqual([
+                (merged_key,),
+                (merged_key,),
+                (merged_key,),
+                (merged_key,),
+                ], origins)
+        self.assertRaises(RevisionNotPresent,
+            files.get_annotator().annotate, self.get_simple_key('missing-key'))
+
     def test_get_parent_map(self):
         files = self.get_versionedfiles()
         if self.key_length == 1:
@@ -2637,8 +2663,8 @@
         self.assertRaises(NotImplementedError,
                 self.texts.add_mpdiffs, [])
 
-    def test_check(self):
-        self.assertTrue(self.texts.check())
+    def test_check_noerrors(self):
+        self.texts.check()
 
     def test_insert_record_stream(self):
         self.assertRaises(NotImplementedError, self.texts.insert_record_stream,

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2009-08-04 11:40:59 +0000
+++ bzrlib/ui/text.py	2009-08-06 02:23:37 +0000
@@ -27,6 +27,7 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
+    debug,
     progress,
     osutils,
     symbol_versioning,
@@ -221,6 +222,7 @@
         self._last_task = None
         self._total_byte_count = 0
         self._bytes_since_update = 0
+        self._fraction = 0
 
     def _show_line(self, s):
         # sys.stderr.write("progress %r\n" % s)
@@ -245,9 +247,14 @@
             cols = 20
             if self._last_task is None:
                 completion_fraction = 0
+                self._fraction = 0
             else:
                 completion_fraction = \
                     self._last_task._overall_completion_fraction() or 0
+            if (completion_fraction < self._fraction and 'progress' in
+                debug.debug_flags):
+                import pdb;pdb.set_trace()
+            self._fraction = completion_fraction
             markers = int(round(float(cols) * completion_fraction)) - 1
             bar_str = '[' + ('#' * markers + spin_str).ljust(cols) + '] '
             return bar_str

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2009-07-15 16:52:26 +0000
+++ bzrlib/versionedfile.py	2009-08-04 04:36:34 +0000
@@ -913,7 +913,16 @@
         raise NotImplementedError(self.annotate)
 
     def check(self, progress_bar=None):
-        """Check this object for integrity."""
+        """Check this object for integrity.
+        
+        :param progress_bar: A progress bar to output as the check progresses.
+        :param keys: Specific keys within the VersionedFiles to check. When
+            this parameter is not None, check() becomes a generator as per
+            get_record_stream. The difference to get_record_stream is that
+            more or deeper checks will be performed.
+        :return: None, or if keys was supplied a generator as per
+            get_record_stream.
+        """
         raise NotImplementedError(self.check)
 
     @staticmethod
@@ -1132,10 +1141,15 @@
     def get_annotator(self):
         return annotate.Annotator(self)
 
-    def check(self, progress_bar=None):
+    def check(self, progress_bar=None, keys=None):
         """See VersionedFiles.check()."""
+        # XXX: This is over-enthusiastic but as we only thunk for Weaves today
+        # this is tolerable. Ideally we'd pass keys down to check() and 
+        # have the older VersiondFile interface updated too.
         for prefix, vf in self._iter_all_components():
             vf.check()
+        if keys is not None:
+            return self.get_record_stream(keys, 'unordered', True)
 
     def get_parent_map(self, keys):
         """Get a map of the parents of keys.

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2009-07-23 00:24:30 +0000
+++ bzrlib/workingtree.py	2009-08-04 04:36:34 +0000
@@ -281,6 +281,16 @@
         self._control_files.break_lock()
         self.branch.break_lock()
 
+    def _get_check_refs(self):
+        """Return the references needed to perform a check of this tree.
+        
+        The default implementation returns no refs, and is only suitable for
+        trees that have no local caching and can commit on ghosts at any time.
+
+        :seealso: bzrlib.check for details about check_refs.
+        """
+        return []
+
     def requires_rich_root(self):
         return self._format.requires_rich_root
 
@@ -2536,12 +2546,17 @@
         return un_resolved, resolved
 
     @needs_read_lock
-    def _check(self):
+    def _check(self, references):
+        """Check the tree for consistency.
+
+        :param references: A dict with keys matching the items returned by
+            self._get_check_refs(), and values from looking those keys up in
+            the repository.
+        """
         tree_basis = self.basis_tree()
         tree_basis.lock_read()
         try:
-            repo_basis = self.branch.repository.revision_tree(
-                self.last_revision())
+            repo_basis = references[('trees', self.last_revision())]
             if len(list(repo_basis.iter_changes(tree_basis))) > 0:
                 raise errors.BzrCheckError(
                     "Mismatched basis inventory content.")
@@ -2593,6 +2608,10 @@
         if self._inventory is None:
             self.read_working_inventory()
 
+    def _get_check_refs(self):
+        """Return the references needed to perform a check of this tree."""
+        return [('trees', self.last_revision())]
+
     def lock_tree_write(self):
         """See WorkingTree.lock_tree_write().
 
@@ -2655,6 +2674,10 @@
                 mode=self.bzrdir._get_file_mode())
             return True
 
+    def _get_check_refs(self):
+        """Return the references needed to perform a check of this tree."""
+        return [('trees', self.last_revision())]
+
     @needs_tree_write_lock
     def set_conflicts(self, conflicts):
         self._put_rio('conflicts', conflicts.to_stanzas(),

=== added file 'doc/developers/check.txt'
--- doc/developers/check.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/check.txt	2009-05-12 03:50:39 +0000
@@ -0,0 +1,63 @@
+Check Notes
+===========
+
+.. contents:: :local:
+
+Overview
+--------
+
+Check has multiple responsibilities:
+
+* Ensure that the data as recorded on disk is accessible intact and unaltered.
+* Ensure that a branch/repository/tree/whatever is ready for upgrade.
+* Look for and report on recorded-data issues where previous bzr's, or changing
+  situations have lead so some form of inconsistency.
+* Report sufficient information for a user to either fix the issue themselves
+  or report a bug that will hopefully be sufficiently detailed we can fix based
+  on the initial report.
+* Not scare users when run if everything is okey-dokey.
+
+Ideally one check invocation can do all these things.
+
+Repository
+----------
+
+Things that can go wrong:
+* Bit errors or alterations may occur in raw data.
+* Data that is referenced may be missing
+* There could be a lot of garbage in upload etc.
+* File graphs may be inconsistent with inventories and parents.
+* The revision graph cache can be inconsistent with the revision data.
+
+Branch
+------
+
+Things that can go wrong:
+* Tag or tip revision ids may be missing from the repo.
+* The revno tip cache may be wrong.
+* Various urls could be problematic (not inaccessible, just invalid)
+* Stacked-on branch could be inaccessible.
+
+Tree
+----
+
+Things that can go wrong:
+* Bit errors in dirstate.
+* Corrupt or invalid shelves.
+* Corrupt dirstates written to disk.
+* Cached inventories might not match repository.
+
+Duplicate work
+--------------
+
+If we check every branch in a repo separately we will encounter duplicate
+effort in assessing things like missing tags/tips, revno cache etc.
+
+Outline of approach
+-------------------
+
+To check a repository, we scan for branches, open their trees and generate
+summary data. We then collect all the summary data in as compact a form as
+possible and do a detailed check on the repository, calling back out to branch
+and trees as we encounter the actual data that that tree/branch requires to
+perform its check.

